/*
 Copyright (c) 2008-2010, VVimaging, Inc. (VVI). All Rights Reserved. Use of this software is governed by the Vvidget Software License Agreement. This software is protected by copyright and licenses restricting use, copying, distribution, and decompilation.
 */

#include <CoreFoundation/CoreFoundation.h>
#include <CoreServices/CoreServices.h>
#include <QuickLook/QuickLook.h>
#include <Cocoa/Cocoa.h>
#include <VvidgetCode/VvidgetCode.h>

#define VV_PRINT_MESSAGES 0U

/* -----------------------------------------------------------------------------
 Generate a preview for file
 
 This function's job is to create preview for designated file
 ----------------------------------------------------------------------------- */

OSStatus GeneratePreviewForURL(void *thisInterface, QLPreviewRequestRef preview, CFURLRef url, CFStringRef contentTypeUTI, CFDictionaryRef options)
{
	
	UInt8 filePath[1024];
	NSAutoreleasePool *pool;
	
#if VV_PRINT_MESSAGES == 1U
	
	fprintf(stderr, "====> Called LineGraphQuickLook GeneratePreviewForURL()\n");
	
#endif
		
	pool = [[NSAutoreleasePool alloc] init];
	
	CFURLGetFileSystemRepresentation(url, TRUE, filePath, 1024);
	
	VC_Client_State *client_state;
	VC_Input_String_Constructor *input_string_constructor;
	NSData *image_data;
	unsigned width, height;
	unsigned image_output_type;
	
	image_output_type = 1U;
	width = 800U;
	height = 600U;
	
	client_state = [[VC_Client_State alloc] init];
	input_string_constructor = [client_state get_VC_input_string_constructor]; // by reference only
	
	[input_string_constructor VC_reset];
	
	[input_string_constructor VC_append_key:"pvs_version" unsigned_value:1U];
	
	[input_string_constructor VC_append_key:"vvidgetcode_deployment_license" string_value:@"embedded 1d65ch78f09aadR06464fY6"];
	
	[input_string_constructor VC_append_key:"image_maker" string_value:@"chart"];
	[input_string_constructor VC_append_key:"chart_type" string_value:@"line"];
	[input_string_constructor VC_append_key:"chart_subtype" string_value:@"linear"];
	[input_string_constructor VC_append_key:"chart_format_type" string_value:@"default"];
	
	[input_string_constructor VC_append_key:"title" string_value:@"Line Graph Quick Look"];
	[input_string_constructor VC_append_key:"x_title" string_value:@"X"];
	[input_string_constructor VC_append_key:"y_title" string_value:@"Y"];
	
	[input_string_constructor VC_append_key:"data_format_type" unsigned_value:0U];
	[input_string_constructor VC_append_key:"data_reference_type" unsigned_value:1U];
	[input_string_constructor VC_append_key:"data_values" string_value:[NSString stringWithUTF8String:filePath]];
	
	[input_string_constructor VC_append_key:"image_width" unsigned_value:width];
	[input_string_constructor VC_append_key:"image_height" unsigned_value:height];
	[input_string_constructor VC_append_key:"image_output_type" unsigned_value:image_output_type];
		
	[client_state VC_update_using_host_string:@"localhost"];
	[client_state set_VC_image_output_type:image_output_type];
	[client_state set_VC_image_width:width];
	[client_state set_VC_image_height:height];
	//MAKES ERROR: [client_state set_VC_seconds_to_timeout:20U];

	[client_state VC_update_using_input_string_constructor];

	[client_state VC_form_query_data];
	//[client_state VC_increment_building_image_request_count];
	
	image_data = [client_state VC_output_data_from_query_data:[client_state get_VC_query_data]];
	
	[client_state VC_invalidate_connection];
	
	if(image_data != nil)
	{
		QLPreviewRequestSetDataRepresentation(preview, (CFDataRef)image_data, kUTTypeImage, NULL);
		
		[image_data release];
	}
	
	[client_state release];
	[pool release];
	
    return noErr;
}

void CancelPreviewGeneration(void* thisInterface, QLPreviewRequestRef preview)
{
    // implement only if supported
}
